@@ -7,6 +7,10 @@ use std::fs::File;
 use std::io::{BufReader, Read, Write};
 use std::path::{Path, PathBuf};
 
+mod serialization;
+
+type Vocab = HashMap<String, u32>;
+
 #[derive(Debug)]
 pub enum Error {
     MissingUnkToken,
@@ -82,20 +86,28 @@ impl WordLevelBuilder {
     }
 }
 
+#[derive(PartialEq, Clone)]
 pub struct WordLevel {
     vocab: HashMap<String, u32>,
     vocab_r: HashMap<u32, String>,
     unk_token: String,
 }
 
+impl std::fmt::Debug for WordLevel {
+    fn fmt(&self, fmt: &mut std::fmt::Formatter) -> std::fmt::Result {
+        fmt.debug_struct("WordLevel")
+            .field("unk_token", &self.unk_token)
+            .field("vocab", &self.vocab.len())
+            .finish()
+    }
+}
+
 impl WordLevel {
-    fn builder() -> WordLevelBuilder {
+    pub fn builder() -> WordLevelBuilder {
         WordLevelBuilder::new()
     }
 
-    /// Initialize a WordLevel model from vocab and merges file.
-    pub fn from_files(vocab_path: &str, unk_token: String) -> Result<WordLevel> {
-        // Read vocab.json
+    pub fn read_file(vocab_path: &str) -> Result<Vocab> {
         let vocab_file = File::open(vocab_path)?;
         let mut vocab_file = BufReader::new(vocab_file);
         let mut buffer = String::new();
@@ -115,7 +127,12 @@ impl WordLevel {
             }
             _ => return Err(Box::new(Error::BadVocabulary)),
         };
+        Ok(vocab)
+    }
 
+    /// Initialize a WordLevel model from vocab and merges file.
+    pub fn from_file(vocab_path: &str, unk_token: String) -> Result<WordLevel> {
+        let vocab = WordLevel::read_file(vocab_path)?;
         Ok(Self::builder().vocab(vocab).unk_token(unk_token).build())
     }
 }
@@ -131,33 +148,24 @@ impl Default for WordLevel {
 }
 
 impl Model for WordLevel {
-    fn tokenize(&self, tokens: Vec<(String, (usize, usize))>) -> Result<Vec<Token>> {
-        let mut output_tokens = vec![];
-
-        for (index, (token, initial_offsets)) in tokens.into_iter().enumerate() {
-            let t = Token {
-                id: *self
-                    .vocab
-                    .get(&*token)
-                    .or_else(|| self.vocab.get(&*self.unk_token))
-                    .ok_or(Error::MissingUnkToken)?,
-                value: token,
-                offsets: initial_offsets,
-                word: index as u32,
-            };
-
-            output_tokens.push(t);
-        }
-
-        Ok(output_tokens)
+    fn tokenize(&self, token: &str) -> Result<Vec<Token>> {
+        Ok(vec![Token {
+            id: *self
+                .vocab
+                .get(&*token)
+                .or_else(|| self.vocab.get(&*self.unk_token))
+                .ok_or(Error::MissingUnkToken)?,
+            value: token.to_owned(),
+            offsets: (0, token.len()),
+        }])
     }
 
     fn token_to_id(&self, token: &str) -> Option<u32> {
         self.vocab.get(token).copied()
     }
 
-    fn id_to_token(&self, id: u32) -> Option<String> {
-        self.vocab_r.get(&id).cloned()
+    fn id_to_token(&self, id: u32) -> Option<&str> {
+        self.vocab_r.get(&id).map(String::as_ref)
     }
 
     fn get_vocab(&self) -> &HashMap<String, u32> {
