@@ -47,21 +47,32 @@ impl Encoding {
         }
     }
 
+    pub fn with_capacity(len: usize) -> Self {
+        Encoding {
+            ids: Vec::with_capacity(len),
+            type_ids: Vec::with_capacity(len),
+            tokens: Vec::with_capacity(len),
+            words: Vec::with_capacity(len),
+            offsets: Vec::with_capacity(len),
+            special_tokens_mask: Vec::with_capacity(len),
+            attention_mask: Vec::with_capacity(len),
+            overflowing: vec![],
+        }
+    }
+
     pub fn from_tokens(tokens: Vec<Token>, type_id: u32) -> Self {
         let length = tokens.len();
-        let (ids, tokens, offsets, words) = tokens.into_iter().fold(
+        let (ids, tokens, offsets) = tokens.into_iter().fold(
             (
                 Vec::with_capacity(length),
                 Vec::with_capacity(length),
                 Vec::with_capacity(length),
-                Vec::with_capacity(length),
             ),
-            |(mut ids, mut tokens, mut offsets, mut words), t| {
+            |(mut ids, mut tokens, mut offsets), t| {
                 ids.push(t.id);
                 tokens.push(t.value);
                 offsets.push(t.offsets);
-                words.push(Some(t.word));
-                (ids, tokens, offsets, words)
+                (ids, tokens, offsets)
             },
         );
 
@@ -69,7 +80,7 @@ impl Encoding {
             ids,
             tokens,
             offsets,
-            words,
+            words: vec![None; length],
             type_ids: vec![type_id; length],
             attention_mask: vec![1; length],
             special_tokens_mask: vec![0; length],
@@ -133,6 +144,17 @@ impl Encoding {
         std::mem::replace(&mut self.overflowing, vec![])
     }
 
+    pub(crate) fn process_tokens_with_offsets_mut<F>(&mut self, func: F)
+    where
+        F: FnMut((usize, (&String, &mut Offsets))),
+    {
+        self.tokens
+            .iter()
+            .zip(self.offsets.iter_mut())
+            .enumerate()
+            .for_each(func)
+    }
+
     /// Get the encoded tokens corresponding to the word at the given index in the input sequence,
     /// with the form (start_token, end_token + 1)
     pub fn word_to_tokens(&self, word: u32) -> Option<(usize, usize)> {
@@ -197,13 +219,17 @@ impl Encoding {
 
     /// Truncate the current `Encoding`.
     ///
-    /// Panic if `stride >= max_len` or `max_len == 0`.
+    /// Panic if `stride >= max_len`
     pub fn truncate(&mut self, max_len: usize, stride: usize) {
         if max_len >= self.ids.len() {
             return;
         }
-        // We only truncate if max_len > 0, it makes no sense otherwise
-        assert!(max_len > 0);
+
+        if max_len == 0 {
+            let o = std::mem::replace(self, Encoding::with_capacity(0));
+            self.overflowing.push(o);
+            return;
+        }
 
         // Get the main overflowing part
         let o_ids = self.ids.split_off(max_len);
@@ -276,19 +302,14 @@ impl Encoding {
     }
 
     /// Merge all Encodings together
-    pub fn merge(encodings: &[Encoding], growing_offsets: bool) -> Encoding {
-        if encodings.is_empty() {
-            return Encoding::default();
+    pub fn merge<I: IntoIterator<Item = Encoding>>(encodings: I, growing_offsets: bool) -> Self {
+        let mut encoding = Encoding::default();
+
+        for sub in encodings {
+            encoding.merge_with(sub, growing_offsets);
         }
 
-        let (firsts, others) = encodings.split_at(1);
-        let mut first: Encoding = firsts[0].clone();
-
-        for encoding in others {
-            first.merge_with(encoding.clone(), growing_offsets);
-        }
-
-        first
+        encoding
     }
 
     /// Merge ourself with the given `Encoding`. Happens in place.
@@ -322,20 +343,7 @@ impl Encoding {
         self.ids.extend(pair.ids);
         self.type_ids.extend(pair.type_ids);
         self.tokens.extend(pair.tokens);
-
-        let starting_word = self
-            .words
-            .iter()
-            .filter(|w| w.is_some())
-            .map(|w| w.unwrap())
-            .max()
-            .map_or(0, |w| w + 1);
-        self.words.extend(
-            pair.words
-                .into_iter()
-                .map(|w| w.map(|w| w + starting_word))
-                .collect::<Vec<_>>(),
-        );
+        self.words.extend(pair.words);
 
         let starting_offset = if growing_offsets {
             self.offsets.last().map_or(0, |o| o.1)
@@ -418,6 +426,35 @@ impl Encoding {
     }
 }
 
+impl std::iter::FromIterator<Encoding> for Encoding {
+    fn from_iter<I: IntoIterator<Item = Encoding>>(iter: I) -> Self {
+        Self::merge(iter, false)
+    }
+}
+
+impl std::iter::FromIterator<(u32, String, (usize, usize), Option<u32>, u32)> for Encoding {
+    fn from_iter<I: IntoIterator<Item = (u32, String, (usize, usize), Option<u32>, u32)>>(
+        iter: I,
+    ) -> Self {
+        let items = iter.into_iter();
+        let (lower, upper) = items.size_hint();
+        let length = upper.unwrap_or(lower);
+        let mut encoding = Self::with_capacity(length);
+
+        for (id, token, offsets, word, type_id) in items {
+            encoding.ids.push(id);
+            encoding.tokens.push(token);
+            encoding.offsets.push(offsets);
+            encoding.type_ids.push(type_id);
+            encoding.words.push(word);
+            encoding.special_tokens_mask.push(0);
+            encoding.attention_mask.push(1);
+        }
+
+        encoding
+    }
+}
+
 #[inline]
 fn get_current_part<T: Clone>(
     prev: &[T],
@@ -469,7 +506,7 @@ mod tests {
                 ids: vec![1, 2],
                 type_ids: vec![0, 1],
                 tokens: vec![String::from("Hello "), String::from("World!")],
-                words: vec![Some(0), Some(1)],
+                words: vec![Some(0), Some(0)],
                 offsets: vec![(0, 6), (6, 12)],
                 special_tokens_mask: vec![0, 0],
                 attention_mask: vec![1, 1],
@@ -520,6 +557,52 @@ mod tests {
         );
     }
 
+    #[test]
+    fn truncate_to_empty() {
+        let mut a = Encoding {
+            ids: vec![1, 2, 3],
+            type_ids: vec![0, 0, 0],
+            tokens: vec![
+                String::from("Hello"),
+                String::from("World"),
+                String::from("!"),
+            ],
+            words: vec![Some(0), Some(1), Some(2)],
+            offsets: vec![(0, 5), (6, 11), (11, 12)],
+            special_tokens_mask: vec![0, 0, 0],
+            attention_mask: vec![1, 1, 1],
+            overflowing: vec![],
+        };
+        a.truncate(0, 0);
+
+        assert_eq!(
+            a,
+            Encoding {
+                ids: vec![],
+                type_ids: vec![],
+                tokens: vec![],
+                words: vec![],
+                offsets: vec![],
+                special_tokens_mask: vec![],
+                attention_mask: vec![],
+                overflowing: vec![Encoding {
+                    ids: vec![1, 2, 3],
+                    type_ids: vec![0, 0, 0],
+                    tokens: vec![
+                        String::from("Hello"),
+                        String::from("World"),
+                        String::from("!"),
+                    ],
+                    words: vec![Some(0), Some(1), Some(2)],
+                    offsets: vec![(0, 5), (6, 11), (11, 12)],
+                    special_tokens_mask: vec![0, 0, 0],
+                    attention_mask: vec![1, 1, 1],
+                    overflowing: vec![],
+                }]
+            }
+        );
+    }
+
     #[test]
     fn mappings() {
         let encoding = Encoding {
